@@ -41,7 +41,7 @@ extern char *strstr();
 static int nissetup_flag = 0;
 
 
-static int nis_setup(void)
+int nis_setup(void)
 {
     if (nissetup_flag)
 	return 0;
@@ -54,31 +54,8 @@ static int nis_setup(void)
 }
 
 
-int nis_get_default_domain(char **outdomain)
-{
-    char buf[1025];
-    
-    
-    if (nis_setup())
-	return -1;
 
-    if (_nis_config->domainname)
-    {
-	*outdomain = strdup(_nis_config->domainname);
-	return 0;
-    }
-
-    if (getdomainname(buf, sizeof(buf)) == 0)
-    {
-	*outdomain = strdup(buf);
-	return 0;
-    }
-
-    return -1;
-}
-
-
-nis_name nis_parsesearch(nis_name name, int *len, nis_attr **val)
+nis_name nis_splitname(nis_name name, int *len, nis_attr **val)
 {
     nis_name rname;
     char *cp;
@@ -149,83 +126,12 @@ nis_name nis_parsesearch(nis_name name, int *len, nis_attr **val)
     return rname;
 }
 
-nis_name nis_parsename(nis_name name,
-		       char **domain,
-		       int *srchlen,
-		       nis_attr **srchval)
-{
-    static char *pbuf = NULL;
-    int len;
-    char *ddom;
-    
-
-    if (name == NULL)
-	return NULL;
-    
-    len = strlen(name);
-    if (Xalloc(&pbuf, len+1) == NULL)
-	return NULL;
-    
-    strcpy(pbuf, name);
-    
-    if (len == 0)
-	return pbuf;
-
-    name = nis_parsesearch(pbuf, srchlen, srchval);
-    
-    if (name == NULL || (len = strlen(name)) == 0)
-	return NULL;
-
-    strcpy(pbuf, name);
-    if (pbuf[len-1] == '.')
-    {
-	/* Fully specified domainname */
-
-	if (domain)
-	{
-	    char *cp;
-	    int i;
-
-	    
-	    i = sizeof(".org_dir.");
-	    cp = strstr(pbuf, ".org_dir.");
-	    if (cp == NULL)
-	    {
-		i = sizeof(".group_dir.");
-		cp = strstr(pbuf, ".group_dir.");
-	    }
-
-	    if (cp == NULL)
-		*domain = "<unknown>";
-	    else
-		*domain = cp + i - 1;
-	}
-	return pbuf;
-    }
-    
-    if (nis_get_default_domain(&ddom) == 0)
-    {
-	if (Xalloc(&pbuf, len+strlen(ddom)+2) == NULL)
-	    return NULL;
-	
-	strcat(pbuf, ".");
-	strcat(pbuf, ddom);
-	
-	if (domain)
-	    *domain = ddom;
-	
-         return pbuf;
-    }
-    else 
-	return NULL;
-}
 
 
 
-static nis_result *
-  do_niscall(char *domain,
-	     struct nis_result * (*cfh)(void *req, CLIENT *clh),
-	     void *req)
+nis_result *
+  _nis_call(struct nis_result * (*cfh)(void *req, CLIENT *clh),
+	    void *req)
 			     
 {
     struct nis_result *res;
@@ -237,41 +143,99 @@ static nis_result *
     if (nis_setup())
 	return NULL;
 
-    clh = nisconf_domain2client(domain, NULL, &address);
+    clh = nisconf_domain2client(nis_local_directory(), NULL, &address);
     if (clh == NULL)
 	return NULL;
-				
+
     while ((res = (*cfh)(req, clh)) == NULL)
     {
 	nresp_flag = 1;
 	fprintf(stderr,
-    "NIS: server \"%s\" not responding for domain \"%s\", still trying...\n",
-		address,
-		domain);
+    "NIS: server \"%s\" not responding, still trying...\n",
+		address);
     }
 
     if (nresp_flag == 1)
 	fprintf(stderr, "NIS: server \"%s\" OK.\n",
 		address);
 
-    return nis_dupresult(res);
+    return res;
 }
 
 
 nis_result *nis_lookup(nis_name name, u_long flags)
 {
+    nis_result *res;
     struct ns_request nsreq;
-    char *domain;
+    nis_name *names;
+    int i;
 
     
     if (nis_setup())
 	return NULL;
 
-    nsreq.ns_name = nis_parsename(name, &domain, NULL, NULL);
-    nsreq.ns_object.ns_object_len = 0;
-    nsreq.ns_object.ns_object_val = NULL;
+    if (flags & EXPAND_NAME)
+    {
+        names = nis_getnames(name);
+        if (names == NULL)
+            return NULL;
+
+        res = NULL;
+	i = 0;
+	while (names[i] != NULL && (res == NULL || (res->status >= 2)))
+        {
+            nsreq.ns_name = names[i];
+      
+            nsreq.ns_object.ns_object_len = 0;
+            nsreq.ns_object.ns_object_val = NULL;
 	
-    return do_niscall(domain, &nis_lookup_3, &nsreq);
+            res = _nis_call(&nis_lookup_3, &nsreq);
+
+	    i++;
+        }
+
+        if (res && res->status == NIS_NOT_ME)
+            res->status = NIS_NOTFOUND;
+
+    }
+    else
+    {
+        nsreq.ns_name = name;
+      
+	nsreq.ns_object.ns_object_len = 0;
+	nsreq.ns_object.ns_object_val = NULL;
+	
+	res = _nis_call(&nis_lookup_3, &nsreq);
+    }
+
+    return nis_dupresult(res);
+}
+
+
+struct ib_request *_nis_build_ib_request(nis_name name, 
+					 struct ib_request *ibreq)
+{
+    ibreq->ibr_name = nis_splitname(name,
+			 &ibreq->ibr_srch.ibr_srch_len,
+			 &ibreq->ibr_srch.ibr_srch_val);
+    
+    if (ibreq->ibr_name == NULL)
+      return NULL;
+
+    ibreq->ibr_flags = 0;
+    
+    ibreq->ibr_obj.ibr_obj_len = 0;
+    ibreq->ibr_obj.ibr_obj_val = NULL;
+    
+    ibreq->ibr_cbhost.ibr_cbhost_len = 0;
+    ibreq->ibr_cbhost.ibr_cbhost_val = NULL;
+
+    ibreq->ibr_bufsize = 0;
+
+    ibreq->ibr_cookie.n_len = 0;
+    ibreq->ibr_cookie.n_bytes = NULL;
+    
+    return ibreq;
 }
 
 
@@ -279,73 +243,48 @@ nis_result *nis_first_entry(nis_name name)
 {
     struct ib_request ibreq;
     nis_result *res;
-    char *domain;
 
     
     if (nis_setup() == -1)
 	return NULL;
-    
-    ibreq.ibr_name = nis_parsename(name,
-				   &domain,
-				   &ibreq.ibr_srch.ibr_srch_len,
-				   &ibreq.ibr_srch.ibr_srch_val);
-				   
-    ibreq.ibr_flags = 0;
 
-    ibreq.ibr_obj.ibr_obj_len = 0;
-    ibreq.ibr_obj.ibr_obj_val = NULL;
-    
-    ibreq.ibr_cbhost.ibr_cbhost_len = 0;
-    ibreq.ibr_cbhost.ibr_cbhost_val = NULL;
+    if (_nis_build_ib_request(name, &ibreq) == NULL)
+      return NULL;
 
-    ibreq.ibr_bufsize = 0;
-    
-    ibreq.ibr_cookie.n_len = 0;
-    ibreq.ibr_cookie.n_bytes = NULL;
-    
-    res = do_niscall(domain, &nis_ibfirst_3, &ibreq);
+    res = _nis_call(&nis_ibfirst_3, &ibreq);
 
     if (ibreq.ibr_srch.ibr_srch_val)
 	nis_freeattrs(ibreq.ibr_srch.ibr_srch_val,
 		      ibreq.ibr_srch.ibr_srch_len);
 
-    return res;
+    return nis_dupresult(res);
 }
 
 
+
+
 nis_result *nis_next_entry(nis_name name, netobj *cookie)
 {
     nis_result *res;
     struct ib_request ibreq;
-    char *domain;
-    
+
 
     if (nis_setup() == -1)
 	return NULL;
-
-    ibreq.ibr_name = nis_parsename(name,
-				   &domain,
-				   &ibreq.ibr_srch.ibr_srch_len,
-				   &ibreq.ibr_srch.ibr_srch_val);
-    ibreq.ibr_flags = 0;
     
-    ibreq.ibr_obj.ibr_obj_len = 0;
-    ibreq.ibr_obj.ibr_obj_val = NULL;
-    
-    ibreq.ibr_cbhost.ibr_cbhost_len = 0;
-    ibreq.ibr_cbhost.ibr_cbhost_val = NULL;
+    if (_nis_build_ib_request(name, &ibreq) == NULL)
+      return NULL;
 
-    ibreq.ibr_bufsize = 0;
-    
-    ibreq.ibr_cookie = *cookie;
+    if (cookie)
+      ibreq.ibr_cookie = *cookie;
 
-    res = do_niscall(domain, &nis_ibnext_3, &ibreq);
+    res = _nis_call(&nis_ibnext_3, &ibreq);
 
     if (ibreq.ibr_srch.ibr_srch_val)
 	nis_freeattrs(ibreq.ibr_srch.ibr_srch_val,
 		      ibreq.ibr_srch.ibr_srch_len);
 
-    return res;
+    return nis_dupresult(res);
 }
 
 #endif /* ENABLE_NISEMU */
