@@ -33,27 +33,88 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <ctype.h>
+#include <sys/socket.h>
+#include <sys/param.h>
+#include <netinet/in.h>
 #include "yp_misc.h"
 #include "yp_networks.h"
 #include "rpcsvc/ypclnt.h"
-#include "xalloc.h"
-
-
-static char *pbuf = NULL;
 
 
 static struct netent *netent_parse(char *str, int len)
 {
-    static struct netent netd;
-    extern char *strtok();
-
-    
-    if (Xalloc(&pbuf, len) == NULL)
+    static struct netent net;
+    static char buf[2048], temp[2048], nname[YP_MAX_NET_NAME_LEN];
+    static char *alias_list[YP_MAX_NET_ALIASES];
+    static int i, j, shiftcount;
+    static unsigned long addr;
+            
+    /* Can't handle this -> let's just forget it... */
+    if (len > 2040)
 	return NULL;
 
-    strncpy(pbuf, str, len);
+    strncpy(buf, str, len);
+
+    /*
+     * Copy out name
+     */
+    net.n_name = nname;
+    for (i = 0 ; (!(isspace(buf[i])) && (i < len)) ; ++i)
+    {
+       net.n_name[i] = buf[i];
+    }
+    net.n_name[i] = '\0';
+    
+    
+    /*
+     * Eat white space
+     */
+    for ( ; ((isspace(buf[i])) && (i < len)) ; ++i)
+       ;
+
+#ifdef LSB
+    /* 
+     * Copy and build net address
+     */
+    addr       = 0;
+    shiftcount = 24;
+    while (1)
+    {
+       for (j = 0; ((isdigit(buf[i])) && (j < 3)) ; ++i, ++j)
+	  temp[j] = buf[i];
+       temp[j] = '\0';
+       
+       addr |= (atoi(temp) << shiftcount);
+
+       if ((buf[i] != '.') || (shiftcount <= 0))
+	  break;
+       else
+	  ++i;
+
+       shiftcount -= 8;
+    }
+
+    net.n_net = htonl(addr);
+#else
+#error No MSB support, yet.
+#endif
+
+    net.n_addrtype = AF_INET;
+    net.n_aliases  = alias_list;
+
+    j = 0;
+    net.n_aliases[j] = _yp_strip_names(_yp_xstrtok(buf+i, ' '));
+    while ((net.n_aliases[j] = _yp_strip_names(_yp_xstrtok(NULL, ' '))))
+    {
+       if (strlen(net.n_aliases[j]) > 0)
+	   ++j;
+    }
+    net.n_aliases[j] = 0;
+
+    
+    return &net;
     
-    return NULL;
 }
 
 
@@ -136,7 +197,7 @@ struct netent *yp_getnetbyaddr(long net, int type, void *opts)
     char *domain;
     char *result;
     int len;
-    char buf[16];
+    char buf[256];
 
 
     map = _ypopts_getmd(opts, "networks", ".byaddr", &domain);
