@@ -26,18 +26,24 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <linux/socket.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 #include "yp_networks.h"
+#include "yp_misc.h"
+
 extern void print_network(struct netent *);
 
 
 int
 main(int argc, char *argv[])
 {
-   struct network *net;
+   struct netent *net;
+   unsigned long test;
+      
+   if (argc >= 2)
+      test = htonl(_yp_readaddr(argv[1]));
    
-
-    net = yp_getnetbyaddr(atoi(argv[1]), AF_INET, NULL);
+    net = yp_getnetbyaddr(test, AF_INET, NULL);
 
     if (net)
 	print_network(net);
