@@ -25,18 +25,23 @@
 */
 
 #include <stdio.h>
-#include <linux/socket.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 #include <netdb.h>
+#include "yp_networks.h"
+#include "yp_misc.h"
 
 
 void print_network(struct netent *net)
 {
    int member_index;
       
-   printf("%s %d %x",
+   printf("%s\t%s\t%s\t",
 	  net->n_name,
-	  net->n_addrtype,
-	  net->n_net);
+	  net->n_addrtype == AF_INET ? "AF_INET" : "UNKNOWN",
+	  _yp_printaddr(ntohl(net->n_net)));
+   
+
    
    for (member_index = 0 ;
 	net->n_aliases && net->n_aliases[member_index] ;
